"use strict";
exports.__esModule = true;
var utils_1 = require("./utils");
var fs_1 = require("fs");
var semver_1 = require("semver");
var child_process_1 = require("child_process");
var root = process.cwd();
function main() {
    var tags = [];
    var tagsPath = root + "/.git/refs/tags";
    var changeLogPath = utils_1.fsExistsSync(root + "/CHANGELOG.md")
        ? root + "/CHANGELOG.md"
        : root + "/History.md";
    var compare = function (v1, v2) {
        v1 = semver_1.clean(v1);
        v2 = semver_1.clean(v2);
        if (!v1 || !v2) {
            return;
        }
        return semver_1.rcompareIdentifiers(v1, v2);
    };
    try {
        var tagsList = fs_1.readdirSync(tagsPath);
        tagsList = tagsList.sort(compare);
        var versions_1 = [];
        tagsList.reduce(function (prev, next) {
            versions_1.push({ name: prev, diff: next + "..." + prev });
            return next;
        });
        var firstCommitBuffer = child_process_1.execSync('git rev-list --max-parents=0 HEAD');
        var firstCommitString = utils_1.bufferToString(firstCommitBuffer).trim();
        versions_1.push({
            name: tagsList[tagsList.length - 1],
            diff: firstCommitString + "..." + tagsList[tagsList.length - 1]
        });
        var changeLogs_1 = [];
        versions_1.forEach(function (version) {
            var name = version.name, diff = version.diff;
            var logBuffer = child_process_1.execSync("git log --pretty=\"- %s\" " + diff);
            var logString = utils_1.bufferToString(logBuffer);
            var logArray = logString.split('\n');
            logArray.unshift("## " + name);
            changeLogs_1 = changeLogs_1.concat(logArray);
        });
        fs_1.writeFileSync(changeLogPath, changeLogs_1.join('\n'));
    }
    catch (error) {
        throw error;
    }
}
exports["default"] = main;
